#pragma rtGlobals=3		// Use modern global access method and strict wave access.


Menu "Averages"
"SplitMovie"
"CreateDataFolder"
"WeightedAv"
"DisplayResults"
"DSplot"
"Plot"
End

///////////////////////////////////////////////////////////////////////------------------------------------------------------///////////////////////////////////////////////////////////////////////

Proc SplitMovie()
	AV_SplitMovie()
End
Proc CreateDataFolder()
	AV_dataFolder()
End

Proc WeightedAv()
	AV_WeightedAverages()
End

Proc DisplayResults()
	AV_displRes()
End

Proc DSplot()
	AV_DSplot()
End

Proc Plot()
	AV_plot()
End
///////////////////////////////////////////////////////////////////////------------------------------------------------------///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////------------------------------------------------------///////////////////////////////////////////////////////////////////////

Function AV_SplitMovie()

	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr = SIS_getImageStackName()

	wave stk = $stackStr
	
	Variable ss = DimSize(stk, 2) //stack size
	ss = ss/2
	//Debug
	Print "stackStr", stackStr, "Size of each Stack "
	
	//Make the two waves to store separately the results
	Make/N=(DimSize(stk, 0), DimSize(stk, 1), ss)
	
End

///////////////////////////////////////////////////////////////////////------------------------------------------------------///////////////////////////////////////////////////////////////////////

Function AV_dataFolder() //display results
variable n = 4 //number of point of ed waves

String dfname
Variable datafolder
prompt datafolder, "Select from the list", popup "ED;VF;WKF;Graphs;All" // ED: dirac energies, VF: fermi velocities
prompt n, "Number of files:"
doprompt "Create a data folder", datafolder, n


if(V_flag == 0)

		If(datafolder == 1)
			dfname = "ED"
			SUB_AV_createDataFolder (dfname,n)
		endif
		If(datafolder == 2)
			dfname = "VF"
			SUB_AV_createDataFolder (dfname,n)
		endif
		If(datafolder == 3)
			dfname = "WKF"
			SUB_AV_createDataFolder (dfname,n)
		endif
		If(datafolder == 4)
			dfname = "Graphs"
			SUB_AV_createDataFolder (dfname,n)
		endif
		if(datafolder == 5)
			dfname = "ED"
			SUB_AV_createDataFolder (dfname,n)
			dfname = "VF"
			SUB_AV_createDataFolder (dfname,n)
			dfname = "WKF"
			SUB_AV_createDataFolder (dfname,n)
			dfname = "Graphs"
			SUB_AV_createDataFolder (dfname,n)
		endif

Endif
End

///////////////////////////////////////////////////////////////////////------------------------------------------------------///////////////////////////////////////////////////////////////////////

Function AV_displRes() //display results

	String winStr=WinName(0,2)
	
	Variable mode
	
	Variable m1 = StringMatch(winStr, "ED*" )
	if(m1 == 1) //there is a match
		SetDataFolder root:ED
		mode = 1
	endif
	
	Variable m2 = StringMatch(winStr, "VF*" )
	if(m2 == 1)
		SetDataFolder root:VF
		mode = 2
	endif	

	variable m3 = StringMatch(winStr, "WKF*" )
	if(m3 == 1) //there is a match
		SetDataFolder root:WKF
		mode = 3
	endif
	
	string dfStr = GetDataFolder(0)
	string wAvStr, errAvStr //name of the wave which you want to plot
	variable addAv , cleanGr
	string LabelBott = "File Number (#)"
	wave/T filename
	variable cl = 0 // point of the wave with clean results
	
	
	
	prompt wAvStr, "Data Wave", popup WaveList("*",";","TEXT:0")
	prompt errAvStr, "Error Wave", popup WaveList("*err",";","TEXT:0")
	prompt addAv, "Append W. Average??", popup "YES;NO"
	prompt labelBott, "Label of bottom axis"
	prompt cleanGr, "Insert Clean Gr??", popup "YES;NO"
	prompt cl, "Which Point Of the wave has the clean Graphene results??"
	doprompt "Display the results", wAvStr, errAvStr,addAv,cleanGr,cl//, wAvName

if(V_Flag==0)
		
		wave y = $(wAvStr), y_err = $(errAvStr)
		variable dim = dimSize($(wAvStr),0)
		make/O/N=(dim) xw
		variable i,j

		NVAR  wAv = root:$(dfStr):$wAvStr //wAv: weighted average
		variable varEx = NVAR_Exists(wAv)

		if(varEx==0)
			addAv=0
		endif

		j = 1
		for(i=0;i<dim;i+=1)
			xw[i] = j
			j += 1
		endfor

		//wavestats/Q y
		//variable mn = V_min, mx = V_max
		make/O/N=(dim) temp = NaN		
		for(i=0;i<dim;i+=1)
			temp[i] = y[i] + y_err[i]
		endfor
		wavestats/Q temp
		variable ymax = V_max + abs(V_max)*(0.1)

		for(i=0;i<dim;i+=1)
			temp[i] = y[i] - y_err[i]
		endfor
		wavestats/Q temp
		variable ymin = V_min - abs(V_min)*(0.1)
		
		killwaves temp

		string w_name = dfStr + "GraphResults"
		DoWindow/K $(w_name)
		display/K=1/N=$(w_name) y vs xw

		if(addAv==1)
			make/O/N=(dim) ww = wAv //ww:weighted wave, only to put the average on the graph
			appendTograph ww vs xw
			ModifyGraph mode(ww)=0,lstyle(ww)=6,lsize(ww)=1.5,rgb(ww)=(0,0,0)
		endif
		
		ModifyGraph mode($(wAvStr))=3,marker($(wAvStr))=19,msize($(wAvStr))=1.8,lsize($(wAvStr))=1.2,rgb($(wAvStr))=(65280,0,0)
		ModifyGraph width=425,height=198,grid(left)=2
		ErrorBars/T=1.2/L=1.2 $(wAvStr) Y,wave=($(errAvStr),$(errAvStr))

		if(mode == 1)
			Label left "\\f01\\Z14E_d - E_f (eV)"
		endif
		
		if(mode == 2)
			Label left "\\f01\\Z14Fermi Velocity (m/s)"
		endif
		
		if(mode == 3)
			Label left "\\f01\\Z14DeltaWKF (eV)"
		endif
		
		for(i=0;i<dim;i+=1)
			string tb = "text" + num2str(i)
			string tg = filename[i]
			Tag/C/N=$tb/F=2/L=1 $(wAvStr), i, tg
		endfor
	
		LabelBott = "\f01\Z14" + LabelBott
		Label bottom labelBott
		SetAxis left ymin,ymax	
		
		if(cleanGR==1)
			make/N=(dim)/O cGr
			cGr = y[cl]
			AppendToGraph cGr vs xw
			ModifyGraph lstyle(cGr)=3,lsize(cGr)=1.5,rgb(cGr)=(0,0,0)
		endif
		showinfo
	endif


End

///////////////////////////////////////////////////////////////////////------------------------------------------------------///////////////////////////////////////////////////////////////////////

Function AV_WeightedAverages()

	String winStr=WinName(0,2)
	
	Variable match = StringMatch(winStr, "ED*" )
	if(match==1) //there is a match
	SetDataFolder root:ED
	else
	SetDataFolder root:VF
	endif	

	string wAvStr, errAvStr

	prompt wAvStr, "wave that you want to average", popup WaveList("*",";","TEXT:0")
	prompt errAvStr, "wave of the weights", popup WaveList("*err",";","TEXT:0")
	//prompt wAvName, "name of the quantity that you want to average", popup "EDav;VFav"
	doprompt "weighted average", wAvStr, errAvStr//, wAvName

variable Wr, errWr //weighted average result, error on weighted average result

if(V_flag==0)
		duplicate/O $errAvStr tempErrW, tempW
		variable i	
		variable dim = dimSize($wAvStr,0)
		wave e =  $errAvStr
		wave w = $wAvStr
		
		for(i=0;i<dim;i+=1)
			if(e[i] == 0)
				tempErrW[i] = 0
			else
				tempErrW[i] = 1/(e[i])^2
			endif
		endfor
		
		variable sumWeights = sum(tempErrW)

		for(i=0;i<dim;i+=1)
				tempW[i] =  w[i]*tempErrW[i]
		endfor
		
		Wr = sum(tempW)/sumWeights
		errWr = 1/sqrt(sumWeights)	
		KillWaves tempErrW, tempW
		
			//string wAv, errAv

variable/G $(wAvStr) = Wr , $(errAvStr) = errWr //weighted average result, error on weighted average result
print wAvStr + "=", wr, errAvStr + "=", errWr
endif

End

///////////////////////////////////////////////////////////////////////------------------------------------------------------///////////////////////////////////////////////////////////////////////

Function AV_DSplot()

	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	string nameStr = dfStr[5,13]
	variable bgkMode = 2
	
	string cfStr
	Prompt cfStr, "DS Coeffiecients", popup WaveList("*coef*",";","")
	Prompt bgkMode, "Add background", popup "yes;no"
	doPrompt "Select the coefficients wave", cfStr,bgkMode
	
if(V_flag==0)
	wave cofW = $cfStr 
	make/O/n=(400) xw //x wave for results
	variable npeaks = cofW[0], i, j // i and j: sum index
	variable delta = 8/401, incr = 0 // 7eV divided by the number of points of the total wave
	make/O/N=(5) cw_sp //coefficients wave for the single peak
	make/O/N=(npeaks*5 + 2) cw // coefficient wave for the whole spectrum
		
	//I'm creating the x wave
	incr = cofW[9] - 4 // plot in kinetic energies
	for(i=0;i<400;i+=1) 
		xw[i] = incr
		incr += delta
	endfor
	
	//plot the whole spectrum
	string fitStr = "DS_" + nameStr
	make/O/n=(400) $fitStr
	wave w = $(fitStr)
	
		for(j=2; j<5*npeaks + 2;j+=1)
			cw[j] = cofW[5 + j - 2]
		endfor	

		cw[0] = cofW[1]
		cw[1] = cofW[2]
		
		if(npeaks>=2)
			for(i=2; i<=npeaks;i+=1)
				cw[i*5 + 1] = cofW[9] - cofW[i*5 + 4]		
			endfor
			
			// integrals of the curves
			string intVstr =  "intV_" + nameStr // intergal values 
			string intSTstr =  "intST_" + nameStr // intergal ratios
			make/N=(npeaks + 1)/O $(intVstr) // storage the integral values
			make/N=(npeaks)/O $(intSTstr) // storage the integral ratios
			wave intW = $(intVstr), rtw = $(intSTstr)
		endif			
			
		//debug
		string winCoefName = "DScoeff_" + nameStr
		duplicate/O cw dC
		doWindow/K $(winCoefName)
		edit/K=1/N=$(winCoefName) dC
		//make/O/N=(400) fit1
		
		if(bgkMode==1)
			//w = dsgnmEad2_MTHR(cw, xw)
		else 
			DeletePoints 0,2, cw
			//w = dsgnmEad2nb_MTHR(cw, xw)
		endif
		
		string winGraphName = "DSgraph_" + nameStr
		doWindow/K $(winGraphName)
		display/K=1/N=$(winGraphName) w vs xw
		ModifyGraph lsize($fitStr)=1.2,width=360, height={Aspect,1}
		Label left "\\f01\\Z14Intensity (a.u.)", Label bottom "\\f01\\Z14Kinetic Energy (eV)"
		ShowInfo, Legend/A=RT
		
	// integrate all DS
	string intStr = "int_" + nameStr	
	make/O/N=(400) $(intStr)
	integrate/METH=2 $(fitStr) /D=$(intStr)
	wavestats/Q $(intStr)
	String int = "Int_value_" + num2str(i)
		// intW[0] is the value of the total area of the DS without background
	intW[0] = V_max
	wavestats/Q $(fitStr)
	variable loc = V_maxloc
	string tg ="Integral p" + " = " + num2str( intW[0])
	Tag/C/A=LC/N=tx/F=2/L=1 $(fitStr), loc, tg
	
	// plots the single components
	for(i=1;i<=npeaks;i+=1)
			for(j=0; j<5;j+=1)
				cw_sp[j] = cofW[i*5 + j]
			endfor
		
			//debug
			string dbcoeffStr = "dC_" + num2str(i)
			make/O/N=(5) $(dbcoeffStr)
			wave w_deb = $(dbcoeffStr) 
			w_deb = cw_sp
			appendtotable/W=$(winCoefName) $(dbcoeffStr)
			
			fitStr = "fitDS_"+ nameStr + "_" + num2str(i)	
			make/O/n=(400) $fitStr //y wave for results
			wave w = $(fitStr)
			//w = dsgnnb_MTHR(cw_sp, xw)
			appendtograph/W=$(winGraphName) w vs xw
			ModifyGraph lsize($fitStr)=1.2
			ModifyGraph rgb($fitStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
			
			If(npeaks>=2)
				intStr = "int" + nameStr + "_" + num2str(i)	
				make/O/N=(400) $(intStr)
				integrate/METH=2 $(fitStr) /D=$(intStr)
				wavestats/Q $(intStr)
				int = "Int_value_" + num2str(i)
				// intW[0] is the value of the total area of the DS without background
				intW[i] = V_max
				wavestats/Q $(fitStr)
				loc = V_maxloc
				String tb = "text" + num2str(i)
				Variable integral = intW[i]
				tg ="Integral p" + num2str(i)	 + " = " + num2str(integral)
				Tag/C/A=RC/N=$tb/F=2/L=1 $(fitStr), loc, tg
			endif
			
	endfor
	
	If(npeaks>=2)
		for(i=1;i<=npeaks;i+=1)
			rtw[i-1]	=  intW[i]/intW[0] // storage the integral values
		endfor
		String tot ="\rTotal Ratio = " + num2str(rtw[1]*100) + "%"
		string tx = "\f01\Z14" + nameStr + "\r\f00\Z10ratio p2/allDS = " + num2str(rtw[1])
		if(npeaks==3)
			tx = tx + "\r\f00\Z10ratio p3/allDS = " + num2str(rtw[2])
			tot = "\rTotal Ratio = " + num2str((rtw[1] + rtw[2] )*100 ) + "%"
		endif
		tx = tx + tot
		TextBox/C/N=text/A=LT tx
	endif

endif

end

///////////////////////////////////////////////////////////////////////------------------------------------------------------///////////////////////////////////////////////////////////////////////

Function AV_plot()
String yStr,xStr
Variable errModeX = 2, errModeY = 2, lineOn0 = 2

prompt yStr, "y wave name", popup "ED;ChemRatio;WKF_FG"
prompt errModeY, "error on Y", popup "Yes;No"
prompt xStr, "x wave name", popup "WKF_FG;ChemRatio;ED;FileNumber"
prompt errModeX, "error on X", popup "Yes;No"
prompt lineOn0, "append a line on zero", popup "Yes;No"
doprompt "Make the plot", yStr, errModeY, xStr, errModeX,lineOn0

string label1 = "\\f00\\Z14E\\BD\\M\\Z14 Dirac Energy (eV)"
string label2 = "\\f00\\Z14Chemisorption Rate (%)"	
string label3 = "\\f00\\Z14Relative WKF difference (eV)"	
string label4 = "\\f00\\Z14Substrate"	

string labelL,labelB

if(V_flag==0)

		wave xw = $xStr, y = $yStr
		string winGraphName = yStr + "vs"+ xStr
		doWindow/K $(winGraphName)
		display/K=1/N=$(winGraphName) y vs xw
		ModifyGraph mode=3,marker=19,msize=2.5,rgb=(0,0,65280)
		
		if(errModeY==1)
			string errWy = yStr + "_err"
			wave errY = $errWy
			ErrorBars $yStr Y,wave=(errY,errY)
		endif
		
		if(errModeX==1)
			String errWx= xStr + "_err"
			wave errX = $errWx
			ErrorBars $yStr X,wave=(errX,errX)
			if(errModeY==1)
				ErrorBars $yStr XY,wave=(errX,errX), wave=(errY,errY)			
			endif
		endif
		
		Variable m1 = StringMatch(xStr, "Chem*" )
		Variable m2 = StringMatch(xStr, "WK*" )	
		Variable m3 = StringMatch(xStr, "File*" )		
		Variable mx1 = StringMatch(yStr, "WK" )		
	
		labelL= label1		
		
		if(m1==1)
			labelB = label2	
		endif		
		if(m2==1)
			labelB = label3
		endif
		if(m3==1)
			labelB = label4
		endif
		if(mx1==1)
			labelL = label3
		endif
			
		If(lineOn0==1)
			String zeroL = winGraphName + "_zero"
			variable dim = dimSize($xStr,0)
			make/O/N=(dim + 2) $zeroL
			wave zero = $zeroL
			zero = 0
			appendToGraph zero 
			ModifyGraph lstyle($zeroL)=1, lsize($zeroL)=1.2, rgb($zeroL)=(0,0,0)
		endif

		ModifyGraph lblMargin(left)=5,lblMargin(bottom)=3
		Label left labelL, Label bottom labelB
		ShowInfo		
		ModifyGraph mirror=2,minor=1
		wavestats/Q  $xStr
		variable mn = V_min - abs(V_min*0.1)
		variable mx = V_max + abs(V_max*0.1)
		SetAxis bottom mn,mx
		ModifyGraph width=216,height={Aspect,0.8}
		



endif
end

///////=============///////=============///////=====FUNCTIONS=====///////=============///////=============///////

Function SUB_AV_createDataFolder (dfname,n)
string dfname
variable n 

string wName = dfname, wErrName = dfname + "_err"

		NewDataFolder/O/S root:$(dfname) // /S sets the datafolder as a current data folder
		make/T/O/N=(n) FileName
		string winN = dfname + "_Results"
		DoWindow/K $(winN)
		variable match = StringMatch(dfname, "*WKF*")

		if(match == 0)		
			make/O/N=(n) $(wName), $(wErrName)
			edit/K=1/N=$(winN) FileName,$(wName),$(wErrName)
		else
			String wName1 = dfname + "_FG", wName2 = dfname + "_BG"
			String wErrName1 = dfname + "_FG_err", wErrName2 = dfname + "_BG_err"
			make/O/N=(n) $(wName1), $(wErrName1), $(wName2), $(wErrName2)
			make/T/O/N=(n) RealFileName
			edit/K=1/N=$(winN) FileName,RealFileName,$(wName1),$(wErrName1),$(wName2),$(wErrName2)
		endif

End